#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <ascii.h>
#include <numeric.h>
#include <string.h>


#define PROGRAM "CONV"
#define VERSION "1.99b"


/*************************************************************/
/* print usage information                                   */
/*************************************************************/

static void print_usage (char *filename)
{
  fprintf (stderr, "%s %s - convolute ASCII data\n\n", PROGRAM, VERSION);
  fprintf (stderr, "written by  Bernhard Mayer,\n");
  fprintf (stderr, "            Fraunhofer-Institute for Atmospheric Research (IFU)\n");
  fprintf (stderr, "            82467 Garmisch-Partenkirchen, Germany\n");
  fprintf (stderr, "Version %s finished March 18, 1997\n\n", VERSION);
  fprintf (stderr, "Be aware that this program is an beta version! Please report any\n");
  fprintf (stderr, "kind of error to the author, including the error message and the\n");
  fprintf (stderr, "database being processed when the error occured. Thanks!\n\n");
  fprintf (stderr, "USAGE: %s <input file> <convolution file>\n\n", filename);
  fprintf (stderr, "%s will convolute ASCII files with the given convolution\n", PROGRAM);
  fprintf (stderr, "function. For this purpose, the ASCII file is interpolated LINEARELY\n");
  fprintf (stderr, "to the resolution of the convolution file, which MUST be given\n");
  fprintf (stderr, "in equidistant wavelength steps. Output is written to stdout.\n");

}



int main(int argc, char **argv)
{  
  char programname  [FILENAME_MAX+200] = "";
  char specfilename [FILENAME_MAX+200] = "";
  char convfilename [FILENAME_MAX+200] = "";

  int i=0, k=0, status=0;
  
  int rows=0, min_columns=0, max_columns=0;

  double **data=NULL;

  int n_conv=0;

  double *x_spec=NULL, *y_spec=NULL, *x_conv=NULL, *y_conv=NULL;
  double **y_spec_conv=NULL;



  strcpy (programname, argv[0]);

  /* check for command line arguments */
  if (argc != 3)  {
    print_usage (argv[0]);
    return (-1);
  }


  /* get filenames from command line */
  strcpy (specfilename, argv[1]);
  strcpy (convfilename, argv[2]);


/*  fprintf (stderr, " ... reading file %s ...\n", specfilename);*/
  /* read file */
  status = ASCII_file2double (specfilename, 
			      &rows, &max_columns, &min_columns, 
			      &data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, specfilename);
    return status;
  }

/*  fprintf (stderr, " ... counted %d data points\n", rows);*/

  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
  }
  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return 0;
  }


/*  fprintf (stderr, " ... reading file %s ...\n", convfilename);*/
  status = read_2c_file (convfilename, &x_conv, &y_conv, &n_conv);
  
  if (status!=0)  {
    fprintf (stderr, " ... error %d reading file %s\n", status, convfilename);
    return status;
  }


/*  fprintf (stderr, " ... convoluting ...\n");*/


  /* allocate memory for result array */
  y_spec_conv = calloc (min_columns-1, sizeof(double *));

  /* do convolution */
  for (k=1; k<min_columns; k++) {
    x_spec = ASCII_column (data, rows, 0);
    y_spec = ASCII_column (data, rows, k);

    if ((status = int_convolute (x_spec, y_spec, rows,
				 x_conv, y_conv, n_conv,
				 &y_spec_conv[k-1])) < 0)  {
      
      switch (status)  {
      case SPEC_NOT_EQUIDISTANT:
	fprintf (stderr, " ... wavelengths in %s not equidistant!\n", specfilename);
	break;
      case CONV_NOT_EQUIDISTANT:
	fprintf (stderr, " ... wavelengths in %s not equidistant!\n", convfilename);
	break;
      case CONV_NOT_CENTERED:
	fprintf (stderr, " ... no center defined in %s\n", convfilename);
	break;
      case SPEC_CONV_DIFFERENT:
	fprintf (stderr, " ... different wavelength steps in %s and %s!\n", 
		 specfilename, convfilename);
	break;
      default:
	fprintf (stderr, " ... numeric function \"conv\" returned undefined error %d\n", status);
	break;
      }
      
      return status;
    }

    free(x_spec);
    free(y_spec);
  }
    

/*  fprintf (stderr, " ... writing to stdout\n");*/

  x_spec = ASCII_column (data, rows, 0);
    
  for (i=0; i<rows; i++)  {
    fprintf (stdout, "%14.8e  ", x_spec[i]);
    for (k=0; k<min_columns-1; k++) 
      fprintf (stdout, "%14.8e  ", y_spec_conv[k][i]);
    fprintf (stdout, "\n");
  }

  /* free memory  */
  free (x_conv);
  free (y_conv);

  for (k=0; k<min_columns-1; k++)
    free (y_spec_conv[k]);

  free (y_spec_conv);

  return 0;
}

